<?php

use PrestaShop\Module\ConotoxiaPay\Configure\Configurator;
use PrestaShop\Module\ConotoxiaPay\Repository\PaymentRepository;
use PrestaShop\Module\ConotoxiaPay\Repository\RefundRepository;

class ConotoxiaPayNotificationModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();
        try {
            $this->setTemplate(ConotoxiaPay::buildTemplatePath('notification'));
            if ($_SERVER['REQUEST_METHOD'] === 'POST') {
                $input = file_get_contents('php://input');
                /** @var CKPL\Pay\Pay $sdk */
                $sdk = $this->module->initializeConotoxiaPay();
                $notification = $sdk->notification()->getNotification($input);
                if ($notification->isPaymentNotification()) {
                    $this->processIncomingPayment($notification);
                    $this->sendResponse('HTTP/1.1 200 OK', 'OK');
                } elseif ($notification->isRefundNotification()) {
                    $this->processIncomingRefund($notification);
                    $this->sendResponse('HTTP/1.1 200 OK', 'OK');
                } else {
                    $this->sendResponse('HTTP/1.1 400 Bad Request', 'Unsupported notification', 400);
                }
            } else {
                $this->sendResponse('HTTP/1.1 405 Method Not Allowed', 'Method Not Allowed', 405);
            }
        } catch (Exception $e) {
            $this->sendResponse('HTTP/1.1 400 Bad Request', $e->getMessage(), 400);
        }
        ob_flush();
    }

    /**
     * @param CKPL\Pay\Notification\Payment\PaymentNotificationInterface $notification
     *
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    private function processIncomingPayment($notification)
    {
        if ($notification->isCompleted()) {
            $order = new Order($notification->getExternalPaymentId());
            if (OrderState::invoiceAvailable(_PS_OS_PAYMENT_)) {
                $order->setInvoice(true);
            }
            if ($order->getCurrentState() == _PS_OS_OUTOFSTOCK_UNPAID_) {
                $order->setCurrentState(_PS_OS_OUTOFSTOCK_PAID_);
            } else {
                $order->setCurrentState(_PS_OS_PAYMENT_);
            }
        } elseif ($notification->isRejected() || $notification->isCancelled()) {
            $order = new Order($notification->getExternalPaymentId());
            $paymentFailedStatus = Configurator::getPaymentFailedStatus();
            $order->setCurrentState($paymentFailedStatus);
        }
        PaymentRepository::updatePaymentStatus($notification->getPaymentId(), $notification->getStatus());
    }

    /**
     * @param CKPL\Pay\Notification\Refund\RefundNotificationInterface $notification
     *
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    private function processIncomingRefund($notification)
    {
        $order = new Order($notification->getExternalPaymentId());
        $validOrder = ConotoxiaPay::isValidOrder($order->getOrderPayments(), $notification->getPaymentId());
        if (!$validOrder) {
            PrestaShopLogger::addLog('ConotoxiaPay - refund notification problem: Payment ' . $notification->getPaymentId() . ' is not assigned to order ' . $notification->getExternalPaymentId(), 3, null, 'Order', $notification->getExternalPaymentId(), true);
            return;
        }

        if (!RefundRepository::isRefundPresent($notification->getRefundId())) {
            RefundRepository::createRefund((int)$notification->getExternalPaymentId(), $notification->getRefundId(), $notification->getPaymentId(), 'UNKNOWN', -1.0, $order->id_currency);
        }

        RefundRepository::updateRefundStatus($notification->getRefundId(), $notification->getStatus());
        if ($notification->isCompleted() && $notification->isMaxRefundAchieved()) {
            PaymentRepository::setMaxRefundAchieved($notification->getRefundId());
        }
    }

    /**
     * @param string $header
     * @param string $message
     * @param int|null $http_response_code
     */
    private function sendResponse(string $header, string $message, $http_response_code = null)
    {
        header($header, true, $http_response_code);
        echo $message;
        ob_flush();
    }
}